import 'dart:collection';

/// [TreeMapper] is a class that steps through a tree structure and maps the matched objects to a different type.
///
/// [TreeMapper] can step through [LinkedHashMap], [List] and first calls the provided mapper method.
/// When the mapper method returns null the [TreeMapper] checks wheter a node is a list or hash and otherwise throws an exception.
/// When the mapper method returns a object, the node at that point is replaced with the mapped object.
class TreeMapper {
  final dynamic Function(dynamic) _mapper;

  TreeMapper(dynamic Function(dynamic) mapper) : _mapper = mapper;

  /// Map a [object] using the mapper method provided in the constructor.
  dynamic map(dynamic object) {
    dynamic node = _mapper(object);
    if (node != null) {
      return node;
    } // If was able to cast

    if (object is List) {
      // Try whether object is a list
      return _mapList(object);
    }

    if (object is LinkedHashMap) {
      // Try whether object is a hash
      return _mapHash(object);
    }

    throw new Exception("Unable to find mapping match for object: $object");
  }

  List<dynamic> _mapList(List<dynamic> list) {
    return list.map((dynamic item) {
      return map(item);
    }).toList();
  }

  Map<dynamic, dynamic> _mapHash(Map<dynamic, dynamic> hash) {
    return hash.map((dynamic key, dynamic item) {
      return MapEntry(key, map(item));
    });
  }
}
